%Script that performs correlation matching on entire region in image with skipping of three pixels for every match
%Inputs are the two stereo images and the coordinates in the images
%Outputs are the parallax values and the row and column range in the image for the area that has been matched

function [parallax, row_range, col_range] = area_matching(master,slave, XI1, XI2)


%Normalize images
master = master./max(max(master));
slave = slave./max(max(slave));


%Show master image and wait for selection of area to be matched 
figure (99)
imagesc(master)
colormap(gray)
title('Please select the area to be matched') 
h = imrect;
pos = round(getPosition(h));
close figure 99

%Get the column and row range in the area that has been selected 
col_range(1) = pos(1);
col_range(2) = pos(1)+pos(3);
row_range(1) = pos(2);
row_range(2) = pos(2)+pos(4);

parallax = zeros(row_range(2)-row_range(1)+1,col_range(2)-col_range(1)+1);
%parallax = zeros(size(master));

h = waitbar(0,'Matching image points...');
offset = zeros(floor((col_range(2)-col_range(1))/2)+1,1);
offset(2:2:end)=1;

%Run the matching for each pixel in selected area
for col=col_range(1):2:col_range(2)
    
    %Update progress bar for each column that has been matched
    waitbar((col-col_range(1)) /(col_range(2)-col_range(1)))
    

for row=row_range(1)+offset((col-col_range(1))/2 +1):2:row_range(2)
     
    %Find location of each matchted pixel in the slave image 
    [~, col_slave] = point_matching(master,slave, row, col);
    
    X_master = XI1(1, col);
    X_slave = XI2(1, col_slave);
    
    %Make parallax matrix with the same size as the selected area
    parallax(row-row_range(1)+1,col-col_range(1)+1) = X_slave-X_master;
    
    
end

end

close(h)    
figure (98)
imagesc(master)
colormap(gray)
title('The selected area that has been matched')
axis([col_range(1) col_range(2) row_range(1) row_range(2)])

end

