%Script that performs correlation matching on entire region in image with no skipping of pixels
%Outputs are the parallax values and the row and column range in the image for the area that has been matched

% When calculating z values this code assumes rectified images with no 
% shifting so that the difference in the corresponding image points is 
% equal to only to the parallax and not the parallax + baseline. 

% If rectified images have been shifted then the second term(b*scale) of 
% calculating the parallax in meters must be removed. 

function Z_meters = area_matching_noskip(master,slave, XI1, XI2)

b = 0.1;                                                                    %baseline in meters
pix_meters = 5.3 * 10^-6;                                                   %pixels to meters
foc_length = 0.035;                                                         %Focal length in meters
height = 0.3;                                                               %Distance between objet plane and imageplane
scale = foc_length/height;                                                  %Scaling factor in image

%Normalize images
master = master./max(max(master));
slave = slave./max(max(slave));


%Show master image and wait for selection of area to be matched 
figure (99)
imagesc(master)
colormap(gray)
title('Please select the area to be matched') 
h = imrect;
pos = round(getPosition(h));
close figure 99

%Get the column and row range in the area that has been selected 
col_range(1) = pos(1);
col_range(2) = pos(1)+pos(3);
row_range(1) = pos(2);
row_range(2) = pos(2)+pos(4);

parallax = zeros(row_range(2)-row_range(1)+1,col_range(2)-col_range(1)+1);


h = waitbar(0,'Matching image points...');


%Run the matching for each pixel in selected area
for col=col_range(1):col_range(2)
    
    %Update progress bar for each column that has been matched
    waitbar((col-col_range(1)) /(col_range(2)-col_range(1)))
    

for row=row_range(1):row_range(2)
     
    %Find location of each matchted pixel in the slave image 
    [~, col_slave] = point_matching(master,slave, row, col);
    
   % X_master = XI1(1, col);
    
%     col_slave(col_slave<0) = X_master;
%     col_slave(col_slave>size(slave,2)) = X_master;
    
   % X_slave = XI2(1, col_slave);
    
   
    %Make parallax matrix with the same size as the selected area
    parallax(row-row_range(1)+1,col-col_range(1)+1) = col-col_slave; %X_slave-X_master;


end

end

close(h)    


%Remove extreme values
[r_up,c_up] = find(parallax(51:end-51,51:end-51)>360);
[r_dw,c_dw] = find(parallax(51:end-51,51:end-51)<160);

r_up= r_up+51;
c_up = c_up +51;
r_dw = r_dw + 51;
c_dw = c_dw +51;

parallax_new = parallax;


%Calculate new values for the removed extreme values based on the mean in the neighbourhood 
for i=1:numel(r_up)
parallax_new(r_up(i), c_up(i)) = mean(mean((parallax_new(r_up(i)-50:r_up(i)+50,c_up(i)-50:c_up(i)+50))));
end
for i=1:numel(r_dw)
parallax_new(r_dw(i), c_dw(i)) = mean(mean((parallax_new(r_dw(i)-50:r_dw(i)+50,c_dw(i)-50:c_dw(i)+50))));
end



figure (98)
imagesc(master)
colormap(gray)
title('The selected area that has been matched')
axis([col_range(1) col_range(2) row_range(1) row_range(2)])

%Convert parallax to meters and calculate Z-values
parallax_meters = parallax_new.*pix_meters./scale;% + b*scale;
Z_meters = foc_length - foc_length*b./parallax_meters;



end

