%Function that locates and plots corresponding points in to epipolar images
%Master is reference image and corresponding point is located in slave
%image

function [index_row, index_col] = correlation(master,slave, row, col, max_shift)

win_size = 10;                                                                       % Window size: (N+1) x (N+1)

%Normalize images
master = master./max(max(master));
slave = slave./max(max(slave));

win0 = master(row-win_size/2:row+win_size/2,col-win_size/2:col+win_size/2);         % Ref/Master window
E_w0 = mean(mean(win0));                                                            % Mean value of window
std_w0 = sqrt(mean(mean((win0-E_w0).^2)));                                          % Standard devation value

norm_corr_1 = zeros(size(slave,2),1);
norm_corr_2 = norm_corr_1;
norm_corr_3 = norm_corr_1;

for i=0:max_shift+20
    
    start = col-max_shift/2;                                                        % Starting columm for windows
    
    win1 = slave(row-win_size/2-1:row+win_size/2-1,start+i:start+i+win_size);       % Slave window 1
    E_w1 = mean(mean(win1));                                                        % Mean value of window 1
    std_w1 = sqrt(mean(mean((win1-E_w1).^2)));                                      % Standard devation value
    
    win2 = slave(row-win_size/2:row+win_size/2,start+i:start+i+win_size);           % Slave window 2
    E_w2 = mean(mean(win2));                                                        % Mean value of window 2
    std_w2 = sqrt(mean(mean((win2-E_w2).^2)));                                      % Standard devation value
    
    win3 = slave(row-win_size/2+1:row+win_size/2+1,start+i:start+i+win_size);       % Slave window 3
    E_w3 = mean(mean(win3));                                                        % Mean value of window 3
    std_w3 = sqrt(mean(mean((win3-E_w3).^2)));                                      % Standard devation value
    
    % Find the normalized correlation for each window along epipolar line
    % and shift half the window size in element number to adjust for 
    % centering
    norm_corr_1(i+start+win_size/2) = (mean(mean(win0.*win1))-E_w0*E_w1)/(std_w0*std_w1);
    norm_corr_2(i+start+win_size/2) = (mean(mean(win0.*win2))-E_w0*E_w2)/(std_w0*std_w2);
    norm_corr_3(i+start+win_size/2) = (mean(mean(win0.*win3))-E_w0*E_w3)/(std_w0*std_w3);
    
    
end

norm_corr = [norm_corr_1 norm_corr_2 norm_corr_3];

[~,index] = max(max(norm_corr));                                                % Find which row that has max corr.
[~,index_col] = max(norm_corr(:,index));                                        % Fin which col within row that has max corr.

index_row = row+index-2;

%Show the images with points
figure(1);
imagesc(master);
colormap(gray);
title('Master image');
hold on;
plot(col,row, '.r')
hold off;

figure(2);
imagesc(slave);
colormap(gray);
title('Slave image');
hold on;
plot(index_col,index_row, '.g');
hold off;

end