%Script that computes values that assess how successful the calibration have been.
%Input parameters are the image file "kuler.bmp" both in distorted form and undistorted form
%Output parameters are compactness and difference in area in the balls after and before calibration 
% NB! Must have Image Processing Toolbox 8.0 (MATLAB R2012A) or later version to run this script successfully

function [comp_dist, area_dist, comp_udist, area_udist ]=verify_cal(img_dist, img_udist)

% Relative part of the pixel values that are kept
rel_part = 0.65;

%Min. and max. radii
radi_min = 20;
radi_max = 40;

%Crop images with 10 pixels in both directions
img_dist = img_dist(5:end-5,5:end-5);
img_udist = img_udist(5:end-5,5:end-5);

%Adjust pixel values for better contrast
J_dist =  imadjust(img_dist,[0 rel_part], []);
J_udist =  imadjust(uint8(img_udist),[0 rel_part], []);

img_size = size(img_dist);


% NB! Must have Image Processing Toolbox 8.0 (MATLAB R2012A) or later version to run this code  
[CENTERS_dist, RADII_dist] = imfindcircles(J_dist, [radi_min radi_max], 'ObjectPolarity', 'dark'); 
[CENTERS_udist, RADII_udist] = imfindcircles(J_udist, [radi_min radi_max], 'ObjectPolarity', 'dark');


%Locate closest center point by looking at the difference each point has with the
%theoretical center
sum_diff = abs(CENTERS_dist(:,1) - img_size(2)/2) + abs(CENTERS_dist(:,2) - img_size(1)/2);

%Find index of center point
[~, loc_center] = min(abs(sum_diff));

%Lookup radius of center point in image
radi_center = RADII_dist(loc_center);


figure
imagesc(J_dist)
title('Distorted image')

hold on
th = 0:pi/50:2*pi;

%Plot circles over distorted image
for i=1:length(RADII_dist)
    
    x1 = RADII_dist(i) * cos(th) + CENTERS_dist(i,1);
    y1 = RADII_dist(i) * sin(th) + CENTERS_dist(i,2);
    plot(x1, y1, 'r'  );
    text(CENTERS_dist(i,1)+25,CENTERS_dist(i,2)-25,num2str(i));
    
    x2 = radi_center * cos(th) + CENTERS_dist(i,1);
    y2 = radi_center * sin(th) + CENTERS_dist(i,2);
    plot(x2, y2, 'g'  );

    comp_dist(i) = length(x1)^2/(pi*RADII_dist(i)^2);
end
hold off

%Calculate area difference 
area_dist = pi*(radi_center-RADII_dist).^2;


%Locate closest center point by looking at the difference each point has with the
%theoretical center
sum_diff = abs(CENTERS_udist(:,1) - img_size(2)/2) + abs(CENTERS_udist(:,2) - img_size(1)/2);

%Find index of center point
[~, loc_center] = min(abs(sum_diff));

%Lookup radius of center point in image
radi_center = RADII_udist(loc_center);



figure
imagesc(J_udist)
title('Undistorted image')
hold on

%Plot circles over udistorted image
for i=1:length(RADII_udist)
    
    x3 = RADII_udist(i) * cos(th) + CENTERS_udist(i,1);
    y3 = RADII_udist(i) * sin(th) + CENTERS_udist(i,2);
    plot(x3, y3, 'r'  );
    text(CENTERS_udist(i,1)+25,CENTERS_udist(i,2)-25,num2str(i));
    
    x4 = radi_center * cos(th) + CENTERS_udist(i,1);
    y4 = radi_center  * sin(th) + CENTERS_udist(i,2);
    plot(x4, y4, 'g'  );
    
    comp_udist(i) = length(x3)^2/(pi*RADII_udist(i)^2);
    
end
hold off

%Calculate area difference 
area_udist = pi*(radi_center-RADII_udist).^2;


